@@ -1,6 +1,6 @@
-from enum import Enum
+from enum import IntEnum
 
 
-class BufferTarget(Enum):
+class BufferTarget(IntEnum):
     ARRAY_BUFFER = 34962
     ELEMENT_ARRAY_BUFFER = 34963
